<?php
/**
 * DokuWiki Countdown Plugin
 * 
 * Shows a countdown in seconds, minutes, hours, days, weeks,
 * month and years calculated for a given datetime
 *
 * @license    http://error-404.de/blog/software
 * @author     Frank Kosanke <digger@sourcelink.de>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
class syntax_plugin_countdown extends DokuWiki_Syntax_Plugin {
 
    function getInfo(){
        return array(
            'author' => 'Frank Kosanke',
            'email'  => 'digger@sourcelink.de',
            'date'   => '2007-09-12',
            'name'   => 'Countdown',
            'desc'   => 'Shows a countdown, calculated for a given date.',
            'url'    => 'http://www.sourcelink.de/coding/dokuwiki/plugin/countdown',
        );
    }


    /**
     * What kind of syntax are we?
     */
    function getType() {
	return 'substition';
        }
	

    /**
     * Where to sort in?
     */ 
    function getSort() {
	return 999;
        }
 
 
    /**
     * Connect pattern to lexer
     */
    function connectTo($mode) {
	$this->Lexer->addSpecialPattern('<countdown=.+?>', $mode, 'plugin_countdown');
        }
 

    /**
     * Handle the match
     */
    function handle($match, $state, $pos, &$handler) {
        substr($match, 11, -1);
        return array(substr($match, 11, -1));
        }

 
    /**
     * Create output
     */
    function render($mode, &$renderer, $data) {
	$startcolor = "";
	$endcolor = "";

	$str = $data[0];
	// Type ermitteln und entfernen
	$type = substr($str,0,1);
	$str = substr($str,2);

	$starttime = time();
	$endtime = strtotime($str);
	$difftime = $endtime - $starttime;
	if ($endtime < $starttime) {
	    $startcolor = "<span style=\"color:#ff0000;\">";
	    $endcolor = "</span>";
	    }

	switch ($type) {
	case "Y":
	    $result = ceil($difftime / (60*60*24*365));
	    $unit = "Jahr";
	    if (($result != 1) && ($result != -1)) { $unit .= "e"; }
	    break;
	case "M":
	    $result = ceil($difftime / (60*60*24*31));
	    $unit = "Monat";
	    if (($result != 1) && ($result != -1)) { $unit .= "e"; }
	    break;
	case "W":
	    $result = ceil($difftime / (60*60*24*7));
	    $unit = "Woche";
	    if (($result != 1) && ($result != -1)) { $unit .= "n"; }
	    break;
	case "D":
	    $result = ceil($difftime / (60*60*24));
	    $unit = "Tag";
	    if (($result != 1) && ($result != -1)) { $unit .= "e"; }
	    break;
	case "h":
	    $result = ceil($difftime / (60*60));
	    $unit = "Stunde";
	    if (($result != 1) && ($result != -1)) { $unit .= "n"; }
	    break;
	case "m":
	    $result = ceil($difftime / 60);
	    $unit = "Minute";
	    if (($result != 1) && ($result != -1)) { $unit .= "n"; }
	    break;
	case "s":
	    $result = $difftime;
	    $unit = "Sekunden";
	    break;
	}

        $renderer->doc .= $startcolor . $result . "&nbsp;" . $unit . $endcolor;
        return true;
        }

    }
